// Copyright (c) 2020, QuantStack and XVega Contributors
//
// Distributed under the terms of the BSD 3-Clause License.
//
// The full license is in the file LICENSE, distributed with this software.

#ifndef XVEGA_RANDOM_STRING_HPP
#define XVEGA_RANDOM_STRING_HPP

#include "../xvega_config.hpp"
#include <random>

namespace xv
{
    XVEGA_API std::string random_string(int len);
}

#endif
